#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to owners."""

from opencue_proto import host_pb2
from opencue import Cuebot
import opencue.wrappers.deed
import opencue.wrappers.host


class Owner(object):
    """This class contains the grpc implementation related to an Owner."""

    def __init__(self, owner=None):
        self.data = owner
        self.stub = Cuebot.getStub('owner')

    def delete(self):
        """Deletes the owner record."""
        self.stub.Delete(host_pb2.OwnerDeleteRequest(owner=self.data), timeout=Cuebot.Timeout)

    def getDeeds(self):
        """Returns the list of deeds for the owner.

        :rtype:  list<opencue.wrappers.deed.Deed Wrapper>
        :return: the list of deeds associated with this owner
        """
        response = self.stub.GetDeeds(host_pb2.OwnerGetDeedsRequest(owner=self.data),
                                      timeout=Cuebot.Timeout)
        return [opencue.wrappers.deed.Deed(deed) for deed in response.deeds.deeds]

    def getHosts(self):
        """Returns a list of all hosts this owner is responsible for.

        :rtype:  list<opencue.wrappers.host.Host>
        :return: list of hosts owned by this owner
        """
        response = self.stub.GetHosts(host_pb2.OwnerGetHostsRequest(owner=self.data),
                                      timeout=Cuebot.Timeout)
        return [opencue.wrappers.host.Host(host) for host in response.hosts.hosts]

    def getOwner(self, name):
        """Returns an owner by name.

        :type  name: str
        :param name: Name of the owner
        :rtype: opencue.wrappers.owner.Owner
        :return: owner that matches the specified name
        """
        return Owner(self.stub.GetOwner(host_pb2.OwnerGetOwnerRequest(name=name),
                                        timeout=Cuebot.Timeout).owner)

    def setShow(self, show):
        """Sets the show for the owner.

        :type  show: str
        :param show: name of the show
        """
        self.stub.SetShow(host_pb2.OwnerSetShowRequest(owner=self.data, show=show),
                          timeout=Cuebot.Timeout)

    def takeOwnership(self, host):
        """Sets the hosts for the owner.

        :type  host: str
        :param host: the name of the host to take ownership of
        """
        self.stub.TakeOwnership(host_pb2.OwnerTakeOwnershipRequest(owner=self.data, host=host),
                                timeout=Cuebot.Timeout)

    def hostCount(self):
        """Returns the number of hosts owned by this owner.

        :rtype:  int
        :return: number of hosts owned by this owner
        """
        return self.data.host_count

    def id(self):
        """Returns the owner id.

        :rtype: str
        :return: the owner id"""
        return self.data.id

    def name(self):
        """Returns the owner name.

        :rtype:  str
        :return: the owner name"""
        return self.data.name

    def show(self):
        """Returns the name of the show of the owner.

        :rtype:  str
        :return: the name of the show of the owner"""
        return self.data.show



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.search`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import job_pb2
from opencue_proto import host_pb2
import opencue.search


@mock.patch('opencue.cuebot.Cuebot.getStub')
class JobSearchTests(unittest.TestCase):

    def testByOptions(self, getStubMock):
        jobId = 'A0000000-0000-0000-0000-000000000000'
        stubMock = mock.Mock()
        getStubMock.return_value = stubMock

        opencue.search.JobSearch.byOptions(show=['pipe'], match=['v6'])

        stubMock.GetJobs.assert_called_with(
            job_pb2.JobGetJobsRequest(r=job_pb2.JobSearchCriteria(shows=['pipe'], substr=['v6'])),
            timeout=mock.ANY)

        opencue.search.JobSearch.byOptions(id=[jobId])

        stubMock.GetJobs.assert_called_with(
            job_pb2.JobGetJobsRequest(r=job_pb2.JobSearchCriteria(ids=[jobId])),
            timeout=mock.ANY)

    def testBaseSearchHost(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetHosts.return_value = host_pb2.HostGetHostsResponse()
        getStubMock.return_value = stubMock

        hostSearch = opencue.search.HostSearch(substr=['unittest_host'])
        hostSearch.search()

        stubMock.GetHosts.assert_called_with(
            host_pb2.HostGetHostsRequest(r=host_pb2.HostSearchCriteria(substr=['unittest_host'])),
            timeout=mock.ANY)

    def testBaseSearchJob(self, getStubMock):
        stubMock = mock.Mock()
        getStubMock.return_value = stubMock

        jobSearch = opencue.search.JobSearch(show=['pipe'], match=['v6'])
        jobSearch.search()

        stubMock.GetJobs.assert_called_with(
            job_pb2.JobGetJobsRequest(r=job_pb2.JobSearchCriteria(shows=['pipe'], substr=['v6'])),
            timeout=mock.ANY)

    def testRaiseIfNotList(self, getStubMock):
        with self.assertRaises(TypeError):
            opencue.search.raiseIfNotList('user', 'iamnotalist')

        with self.assertRaises(TypeError):
            opencue.search.raiseIfNotList('user', 42)

        with self.assertRaises(TypeError):
            opencue.search.raiseIfNotList('user', {'iamnotalist'})

        self.assertIsNone(opencue.search.raiseIfNotList('user', ['iamnotalist']))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.layer`"""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import getpass
import os
import platform
import unittest

import mock

from opencue_proto import depend_pb2
from opencue_proto import job_pb2
import opencue.wrappers.frame
import opencue.wrappers.layer
import opencue.wrappers.job


TEST_LAYER_NAME = 'testLayer'
TEST_OUTPUT_PATH = '/path/to/file.txt'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class LayerTests(unittest.TestCase):
    """Tests for `opencue.wrappers.layer.Layer`."""

    def testKill(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.KillFrames.return_value = job_pb2.LayerKillFramesResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        username = getpass.getuser()
        pid = os.getpid()
        host_kill = platform.uname()[1]
        reason = "Frames Kill Request"
        layer.kill(username=username, pid=pid, host_kill=host_kill, reason=reason)

        stubMock.KillFrames.assert_called_with(
            job_pb2.LayerKillFramesRequest(layer=layer.data,
                                           username=username,
                                           pid=str(pid),
                                           host_kill=host_kill,
                                           reason=reason), timeout=mock.ANY)

    def testEat(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.EatFrames.return_value = job_pb2.LayerEatFramesResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.eat()

        stubMock.EatFrames.assert_called_with(
            job_pb2.LayerEatFramesRequest(layer=layer.data), timeout=mock.ANY)

    def testRetry(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RetryFrames.return_value = job_pb2.LayerRetryFramesResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.retry()

        stubMock.RetryFrames.assert_called_with(
            job_pb2.LayerRetryFramesRequest(layer=layer.data), timeout=mock.ANY)

    def testMarkdone(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.MarkdoneFrames.return_value = job_pb2.LayerMarkdoneFramesResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.markdone()

        stubMock.MarkdoneFrames.assert_called_with(
            job_pb2.LayerMarkdoneFramesRequest(layer=layer.data), timeout=mock.ANY)

    def testAddLimit(self, getStubMock):
        test_limit_id = 'lll-llll-lll'
        stubMock = mock.Mock()
        stubMock.AddLimit.return_value = job_pb2.LayerAddLimitResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.addLimit(test_limit_id)

        stubMock.AddLimit.assert_called_with(
            job_pb2.LayerAddLimitRequest(layer=layer.data, limit_id=test_limit_id),
            timeout=mock.ANY)

    def testDropLimit(self, getStubMock):
        test_limit_id = 'lll-llll-lll'
        stubMock = mock.Mock()
        stubMock.DropLimit.return_value = job_pb2.LayerDropLimitResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.dropLimit(test_limit_id)

        stubMock.DropLimit.assert_called_with(
            job_pb2.LayerDropLimitRequest(layer=layer.data, limit_id=test_limit_id),
            timeout=mock.ANY)

    def testEnableMemoryOptimizerTrue(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.EnableMemoryOptimizer.return_value = job_pb2.LayerEnableMemoryOptimizerResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.enableMemoryOptimizer(True)

        stubMock.EnableMemoryOptimizer.assert_called_with(
            job_pb2.LayerEnableMemoryOptimizerRequest(layer=layer.data, value=True),
            timeout=mock.ANY)

    def testEnableMemoryOptimizerFalse(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.EnableMemoryOptimizer.return_value = job_pb2.LayerEnableMemoryOptimizerResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.enableMemoryOptimizer(False)

        stubMock.EnableMemoryOptimizer.assert_called_with(
            job_pb2.LayerEnableMemoryOptimizerRequest(layer=layer.data, value=False),
            timeout=mock.ANY)

    def testGetFrames(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetFrames.return_value = job_pb2.LayerGetFramesResponse(
            frames=job_pb2.FrameSeq(frames=[job_pb2.Frame(layer_name=TEST_LAYER_NAME)]))
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        frames = layer.getFrames()

        stubMock.GetFrames.assert_called_with(
            job_pb2.LayerGetFramesRequest(
                layer=layer.data,
                s=opencue.search.FrameSearch.criteriaFromOptions()),
            timeout=mock.ANY)
        self.assertEqual(len(frames), 1)
        self.assertEqual(frames[0].data.layer_name, TEST_LAYER_NAME)

    def testGetOutputPaths(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetOutputPaths.return_value = job_pb2.LayerGetOutputPathsResponse(
            output_paths=[TEST_OUTPUT_PATH])
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        outputPaths = layer.getOutputPaths()

        stubMock.GetOutputPaths.assert_called_with(
            job_pb2.LayerGetOutputPathsRequest(layer=layer.data), timeout=mock.ANY)
        self.assertEqual(len(outputPaths), 1)
        self.assertEqual(outputPaths[0], TEST_OUTPUT_PATH)

    def testSetTags(self, getStubMock):
        tags = ['cloud', 'local']
        stubMock = mock.Mock()
        stubMock.SetTags.return_value = job_pb2.LayerSetTagsResponse()
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.setTags(tags)

        stubMock.SetTags.assert_called_with(
            job_pb2.LayerSetTagsRequest(layer=layer.data, tags=tags), timeout=mock.ANY)

    def testSetMaxCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetMaxCores.return_value = job_pb2.LayerSetMaxCoresResponse()
        getStubMock.return_value = stubMock

        testCores = 100
        testCoresActual = testCores/100.0
        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.setMaxCores(testCores)

        stubMock.SetMaxCores.assert_called_with(
            job_pb2.LayerSetMaxCoresRequest(layer=layer.data, cores=testCoresActual),
            timeout=mock.ANY)

    def testSetMinGpuMemory(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetMinGpuMemory.return_value = job_pb2.LayerSetMinGpuResponse()
        getStubMock.return_value = stubMock

        testCores = 100
        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.setMinGpuMemory(testCores)

        stubMock.SetMinGpuMemory.assert_called_with(
            job_pb2.LayerSetMinGpuMemoryRequest(layer=layer.data, gpu_memory=testCores),
            timeout=mock.ANY)

    def testSetMinMemory(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetMinMemory.return_value = job_pb2.LayerSetMinMemoryResponse()
        getStubMock.return_value = stubMock

        memory = 2048
        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.setMinMemory(memory)

        stubMock.SetMinMemory.assert_called_with(
            job_pb2.LayerSetMinMemoryRequest(layer=layer.data, memory=memory),
            timeout=mock.ANY)

    def testSetThreadable(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetThreadable.return_value = job_pb2.LayerSetThreadableResponse()
        getStubMock.return_value = stubMock

        value = True
        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.setThreadable(value)

        stubMock.SetThreadable.assert_called_with(
            job_pb2.LayerSetThreadableRequest(layer=layer.data, threadable=value),
            timeout=mock.ANY)

    def testGetWhatDependsOnThis(self, getStubMock):
        dependId = 'dddd-ddd-dddd'
        stubMock = mock.Mock()
        stubMock.GetWhatDependsOnThis.return_value = job_pb2.LayerGetWhatDependsOnThisResponse(
            depends=depend_pb2.DependSeq(depends=[depend_pb2.Depend(id=dependId)]))
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        depends = layer.getWhatDependsOnThis()

        stubMock.GetWhatDependsOnThis.assert_called_with(
            job_pb2.LayerGetWhatDependsOnThisRequest(layer=layer.data),
            timeout=mock.ANY)
        self.assertEqual(len(depends), 1)
        self.assertEqual(depends[0].id(), dependId)

    def testGetWhatThisDependsOn(self, getStubMock):
        dependId = 'dddd-ddd-dddd'
        stubMock = mock.Mock()
        stubMock.GetWhatThisDependsOn.return_value = job_pb2.LayerGetWhatThisDependsOnResponse(
            depends=depend_pb2.DependSeq(depends=[depend_pb2.Depend(id=dependId)]))
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        depends = layer.getWhatThisDependsOn()

        stubMock.GetWhatThisDependsOn.assert_called_with(
            job_pb2.LayerGetWhatThisDependsOnRequest(layer=layer.data),
            timeout=mock.ANY)
        self.assertEqual(len(depends), 1)
        self.assertEqual(depends[0].id(), dependId)

    def testCreateDependencyOnJob(self, getStubMock):
        dependId = 'dddd-ddd-dddd'
        jobId = 'jjjj-jjj-jjjj'
        stubMock = mock.Mock()
        stubMock.CreateDependencyOnJob.return_value = job_pb2.LayerCreateDependOnJobResponse(
            depend=depend_pb2.Depend(id=dependId))
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        job = opencue.wrappers.job.Job(
            job_pb2.Job(id=jobId))
        depend = layer.createDependencyOnJob(job)

        stubMock.CreateDependencyOnJob.assert_called_with(
            job_pb2.LayerCreateDependOnJobRequest(layer=layer.data, job=job.data),
            timeout=mock.ANY)
        self.assertEqual(depend.id(), dependId)

    def testCreateDependencyOnLayer(self, getStubMock):
        dependId = 'dddd-ddd-dddd'
        layerId = 'llll-lll-llll'
        stubMock = mock.Mock()
        stubMock.CreateDependencyOnLayer.return_value = job_pb2.LayerCreateDependOnLayerResponse(
            depend=depend_pb2.Depend(id=dependId))
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        dependLayer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(id=layerId))
        depend = layer.createDependencyOnLayer(dependLayer)

        stubMock.CreateDependencyOnLayer.assert_called_with(
            job_pb2.LayerCreateDependOnLayerRequest(layer=layer.data,
                                                    depend_on_layer=dependLayer.data),
            timeout=mock.ANY)
        self.assertEqual(depend.id(), dependId)

    def testCreateDependencyOnFrame(self, getStubMock):
        dependId = 'dddd-ddd-dddd'
        frameId = 'ffff-fff-ffff'
        stubMock = mock.Mock()
        stubMock.CreateDependencyOnFrame.return_value = job_pb2.LayerCreateDependOnFrameResponse(
            depend=depend_pb2.Depend(id=dependId))
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(id=frameId))
        depend = layer.createDependencyOnFrame(frame)

        stubMock.CreateDependencyOnFrame.assert_called_with(
            job_pb2.LayerCreateDependOnFrameRequest(layer=layer.data, frame=frame.data),
            timeout=mock.ANY)
        self.assertEqual(depend.id(), dependId)

    def testCreateFrameByFrameDependency(self, getStubMock):
        dependId = 'dddd-ddd-dddd'
        layerId = 'llll-lll-llll'
        stubMock = mock.Mock()
        stubMock.CreateFrameByFrameDependency.return_value = \
            job_pb2.LayerCreateFrameByFrameDependResponse(depend=depend_pb2.Depend(id=dependId))
        getStubMock.return_value = stubMock

        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        dependLayer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(id=layerId))
        depend = layer.createFrameByFrameDependency(dependLayer)

        stubMock.CreateFrameByFrameDependency.assert_called_with(
            job_pb2.LayerCreateFrameByFrameDependRequest(layer=layer.data,
                                                         depend_layer=dependLayer.data,
                                                         any_frame=False),
            timeout=mock.ANY)
        self.assertEqual(depend.id(), dependId)

    def testRegisterOutputPath(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.RegisterOutputPath.return_value = job_pb2.LayerRegisterOutputPathResponse()
        getStubMock.return_value = stubMock

        outputPath = '/test/output/path'
        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.registerOutputPath(outputPath)

        stubMock.RegisterOutputPath.assert_called_with(
            job_pb2.LayerRegisterOutputPathRequest(layer=layer.data, spec=outputPath),
            timeout=mock.ANY)

    def testReorderFrames(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.ReorderFrames.return_value = job_pb2.LayerReorderFramesResponse()
        getStubMock.return_value = stubMock

        frameRange = '1-10'
        order = job_pb2.REVERSE
        layer = opencue.wrappers.layer.Layer(job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.reorderFrames(frameRange, order)

        stubMock.ReorderFrames.assert_called_with(
            job_pb2.LayerReorderFramesRequest(layer=layer.data, range=frameRange, order=order),
            timeout=mock.ANY)

    def testStaggerFrames(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.StaggerFrames.return_value = job_pb2.LayerStaggerFramesResponse()
        getStubMock.return_value = stubMock

        frameRange = '1-10'
        stagger = 4
        layer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=TEST_LAYER_NAME))
        layer.staggerFrames(frameRange, stagger)

        stubMock.StaggerFrames.assert_called_with(
            job_pb2.LayerStaggerFramesRequest(layer=layer.data, range=frameRange, stagger=stagger),
            timeout=mock.ANY)


class LayerEnumTests(unittest.TestCase):

    def testLayerType(self):
        self.assertEqual(opencue.api.Layer.LayerType.PRE, job_pb2.PRE)
        self.assertEqual(opencue.api.Layer.LayerType.PRE, 0)

    def testOrder(self):
        self.assertEqual(opencue.api.Layer.Order.LAST, job_pb2.LAST)
        self.assertEqual(opencue.api.Layer.Order.LAST, 1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.proc`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import host_pb2
from opencue_proto import job_pb2
import opencue.wrappers.proc


TEST_HOST_NAME = 'testHost'
TEST_JOB_NAME = 'testJob'
TEST_LAYER_NAME = 'testLayer'
TEST_PROC_NAME = 'testProc'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class ProcTests(unittest.TestCase):

    def testKill(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Kill.return_value = host_pb2.ProcKillResponse()
        getStubMock.return_value = stubMock

        proc = opencue.wrappers.proc.Proc(
            host_pb2.Proc(name=TEST_PROC_NAME))
        proc.kill()

        stubMock.Kill.assert_called_with(
            host_pb2.ProcKillRequest(proc=proc.data), timeout=mock.ANY)

    def testUnbook(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Unbook.return_value = host_pb2.ProcUnbookResponse()
        getStubMock.return_value = stubMock

        proc = opencue.wrappers.proc.Proc(
            host_pb2.Proc(name=TEST_PROC_NAME))
        proc.unbook()

        stubMock.Unbook.assert_called_with(
            host_pb2.ProcUnbookRequest(proc=proc.data, kill=False), timeout=mock.ANY)

    def testUnbookKill(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Unbook.return_value = host_pb2.ProcUnbookResponse()
        getStubMock.return_value = stubMock

        proc = opencue.wrappers.proc.Proc(
            host_pb2.Proc(name=TEST_PROC_NAME))
        proc.unbook(kill=True)

        stubMock.Unbook.assert_called_with(
            host_pb2.ProcUnbookRequest(proc=proc.data, kill=True), timeout=mock.ANY)

    def testGetHost(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetHost.return_value = host_pb2.ProcGetHostResponse(
            host=host_pb2.Host(name=TEST_HOST_NAME))
        getStubMock.return_value = stubMock

        proc = opencue.wrappers.proc.Proc(
            host_pb2.Proc(name=TEST_PROC_NAME))
        host = proc.getHost()

        stubMock.GetHost.assert_called_with(
            host_pb2.ProcGetHostRequest(proc=proc.data), timeout=mock.ANY)
        self.assertEqual(host.name(), TEST_HOST_NAME)

    def testGetFrame(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetFrame.return_value = host_pb2.ProcGetFrameResponse(
            frame=job_pb2.Frame(layer_name=TEST_LAYER_NAME))
        getStubMock.return_value = stubMock

        proc = opencue.wrappers.proc.Proc(
            host_pb2.Proc(name=TEST_PROC_NAME))
        frame = proc.getFrame()

        stubMock.GetFrame.assert_called_with(
            host_pb2.ProcGetFrameRequest(proc=proc.data), timeout=mock.ANY)
        self.assertEqual(frame.layer(), TEST_LAYER_NAME)

    def testGetLayer(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetLayer.return_value = host_pb2.ProcGetLayerResponse(
            layer=job_pb2.Layer(name=TEST_LAYER_NAME))
        getStubMock.return_value = stubMock

        proc = opencue.wrappers.proc.Proc(
            host_pb2.Proc(name=TEST_PROC_NAME))
        layer = proc.getLayer()

        stubMock.GetLayer.assert_called_with(
            host_pb2.ProcGetLayerRequest(proc=proc.data), timeout=mock.ANY)
        self.assertEqual(layer.name(), TEST_LAYER_NAME)

    def testGetJob(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetJob.return_value = host_pb2.ProcGetJobResponse(
            job=job_pb2.Job(name=TEST_JOB_NAME))
        getStubMock.return_value = stubMock

        proc = opencue.wrappers.proc.Proc(
            host_pb2.Proc(name=TEST_PROC_NAME))
        job = proc.getJob()

        stubMock.GetJob.assert_called_with(
            host_pb2.ProcGetJobRequest(proc=proc.data), timeout=mock.ANY)
        self.assertEqual(job.name(), TEST_JOB_NAME)


class ProcEnumTests(unittest.TestCase):

    def testRedirectType(self):
        self.assertEqual(opencue.api.Proc.RedirectType.JOB_REDIRECT,
                         host_pb2.JOB_REDIRECT)
        self.assertEqual(opencue.api.Proc.RedirectType.JOB_REDIRECT, 0)

    def testRunState(self):
        self.assertEqual(opencue.api.Proc.RunState.BOOKED, host_pb2.BOOKED)
        self.assertEqual(opencue.api.Proc.RunState.BOOKED, 1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.depend`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import depend_pb2
import opencue.wrappers.depend


TEST_DEPEND_ID = 'zzz-aaa-fff'
TEST_DEPEND_ER = '111-111-111'
TEST_DEPEND_ON = '111-111-111'
TEST_DEPEND_ON_NEG = '222-222-222'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class DependTests(unittest.TestCase):

    def testSatisfy(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Satisfy.return_value = depend_pb2.DependSatisfyResponse()
        getStubMock.return_value = stubMock

        depend = opencue.wrappers.depend.Depend(
            depend_pb2.Depend(id=TEST_DEPEND_ID))
        depend.satisfy()

        stubMock.Satisfy.assert_called_with(
            depend_pb2.DependSatisfyRequest(depend=depend.data), timeout=mock.ANY)

    def testUnsatisfy(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Unsatisfy.return_value = depend_pb2.DependUnsatisfyResponse()
        getStubMock.return_value = stubMock

        depend = opencue.wrappers.depend.Depend(
            depend_pb2.Depend(id=TEST_DEPEND_ID))
        depend.unsatisfy()

        stubMock.Unsatisfy.assert_called_with(
            depend_pb2.DependUnsatisfyRequest(depend=depend.data), timeout=mock.ANY)

    def testIsInternalPos(self, getStubMock):
        stubMock = mock.Mock()
        getStubMock.return_value = stubMock

        dependPos = opencue.wrappers.depend.Depend(
            depend_pb2.Depend(id=TEST_DEPEND_ID,
                              depend_er_job=TEST_DEPEND_ER,
                              depend_on_job=TEST_DEPEND_ON))
        self.assertTrue(dependPos.isInternal())

    def testIsInternalNeg(self, getStubMock):
        stubMock = mock.Mock()
        getStubMock.return_value = stubMock

        dependNeg = opencue.wrappers.depend.Depend(
            depend_pb2.Depend(id=TEST_DEPEND_ID,
                              depend_er_job=TEST_DEPEND_ER,
                              depend_on_job=TEST_DEPEND_ON_NEG))
        self.assertFalse(dependNeg.isInternal())


class DependEnumTests(unittest.TestCase):

    def testDependType(self):
        self.assertEqual(opencue.api.Depend.DependType.JOB_ON_JOB,
                         depend_pb2.JOB_ON_JOB)
        self.assertEqual(opencue.api.Depend.DependType.JOB_ON_JOB, 0)

    def testDependTarget(self):
        self.assertEqual(opencue.api.Depend.DependTarget.ANY_TARGET,
                         depend_pb2.ANY_TARGET)
        self.assertEqual(opencue.api.Depend.DependTarget.ANY_TARGET, 2)


if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to Render Partition."""

from opencue_proto import renderPartition_pb2
from opencue.cuebot import Cuebot


class RenderPartition(object):
    """This class contains the grpc implementation related to a Task."""

    def __init__(self, render_partition=None):
        self.data = render_partition
        self.stub = Cuebot.getStub('renderPartition')

    def delete(self):
        """Deletes the render partition."""
        self.stub.Delete(renderPartition_pb2.RenderPartDeleteRequest(
            render_partition=self.data), timeout=Cuebot.Timeout)


    def setMaxResources(self, cores, memory, gpuMemory, gpuCores):
        """Deletes the render partition."""
        self.stub.SetMaxResources(renderPartition_pb2.RenderPartSetMaxResourcesRequest(
            render_partition=self.data,
            cores=cores,
            memory=memory,
            gpu_memory=gpuMemory,
            gpus=gpuCores
        ), timeout=Cuebot.Timeout)



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to OpenCue allocations."""

from opencue_proto import facility_pb2
from opencue_proto import host_pb2
from opencue.cuebot import Cuebot
import opencue.wrappers.host
import opencue.wrappers.subscription


class Allocation(object):
    """This class contains the grpc implementation related to an Allocation."""

    def __init__(self, allocation=None):
        self.data = allocation
        self.stub = Cuebot.getStub('allocation')

    def delete(self):
        """Deletes the allocation."""
        self.stub.Delete(
            facility_pb2.AllocDeleteRequest(allocation=self.data),
            timeout=Cuebot.Timeout)

    def getHosts(self):
        """Returns the list of hosts for the allocation.

        :rtype:  list<opencue.wrappers.host.Host>
        :return: list of hosts
        """
        hostSeq = self.stub.GetHosts(facility_pb2.AllocGetHostsRequest(allocation=self.data),
                                     timeout=Cuebot.Timeout).hosts
        return [opencue.wrappers.host.Host(h) for h in hostSeq.hosts]

    def getSubscriptions(self):
        """Returns the subscriptions of the allocation.

        :rtype:  list<opencue.wrappers.subscription.Subscription>
        :return: a list of subscriptions
        """
        subscriptionSeq = self.stub.GetSubscriptions(
            facility_pb2.AllocGetSubscriptionsRequest(allocation=self.data),
            timeout=Cuebot.Timeout).subscriptions
        return [opencue.wrappers.subscription.Subscription(sub)
                for sub in subscriptionSeq.subscriptions]

    def reparentHosts(self, hosts):
        """Moves the given hosts into the allocation.

        :type  hosts: list<opencue.wrappers.host.Host>
        :param hosts: the hosts to move to this allocation
        """
        hostSeq = host_pb2.HostSeq()
        # pylint: disable=no-member
        hostSeq.hosts.extend([host.data for host in hosts])
        # pylint: enable=no-member
        self.stub.ReparentHosts(
            facility_pb2.AllocReparentHostsRequest(allocation=self.data, hosts=hostSeq),
            timeout=Cuebot.Timeout)

    def reparentHostIds(self, hostIds):
        """Moves the hosts identified by the given host ids into the allocation.

        :type  hostIds: list<str>
        :param hostIds: the host ids to move to this allocation
        """
        hosts = [opencue.wrappers.host.Host(host_pb2.Host(id=hostId)) for hostId in hostIds]
        self.reparentHosts(hosts)

    def setName(self, name):
        """Sets a new name for the allocation.

        :type  name: str
        :param name: the new name
        """
        self.stub.SetName(
            facility_pb2.AllocSetNameRequest(allocation=self.data, name=name),
            timeout=Cuebot.Timeout)

    def setTag(self, tag):
        """Sets a new tag for the allocation.

        :type  tag: str
        :param tag: the new tag
        """
        self.stub.SetTag(
            facility_pb2.AllocSetTagRequest(allocation=self.data, tag=tag),
            timeout=Cuebot.Timeout)

    def id(self):
        """Returns the id of the allocation.

        :rtype:  str
        :return: the id of the allocation
        """
        return self.data.id

    def name(self):
        """Returns the name of the allocation.

        :rtype:  str
        :return: the name of the allocation
        """
        return self.data.name

    def tag(self):
        """Returns the tag of the allocation.

        :rtype:  str
        :return: the tag of the allocation
        """
        return self.data.tag

    def totalCores(self):
        """Returns the total number of cores in the allocation.

        :rtype:  float
        :return: total number of cores in the allocation
        """
        return self.data.stats.cores

    def totalAvailableCores(self):
        """Returns the total number of cores available for booking in the allocation.

        :rtype:  float
        :return: total number of cores in the allocation
        """
        return self.data.stats.available_cores

    def totalIdleCores(self):
        """Returns the total number of idle cores in the allocation.

        :rtype:  float
        :return: total number of idle cores in the allocation
        """
        return self.data.stats.idle_cores

    def totalRunningCores(self):
        """Returns the total number of running cores in the allocation.

        Each 100 returned is the same as 1 physical core.

        :rtype:  float
        :return: total number of running cores in the allocation
        """
        return self.data.stats.running_cores

    def totalLockedCores(self):
        """Returns the total number of locked cores in the allocation.

        Each 100 returned is the same as 1 physical core.

        :rtype:  float
        :return: total number of locked cores in the allocation
        """
        return self.data.stats.locked_cores

    def totalGpus(self):
        """Returns the total number of gpus in the allocation.

        :rtype:  float
        :return: total number of gpus in the allocation
        """
        return self.data.stats.gpus

    def totalAvailableGpus(self):
        """Returns the total number of gpus available for booking in the allocation.

        :rtype:  float
        :return: total number of gpus in the allocation
        """
        return self.data.stats.available_gpus

    def totalIdleGpus(self):
        """Returns the total number of idle gpus in the allocation.

        :rtype:  float
        :return: total number of idle gpus in the allocation
        """
        return self.data.stats.idle_gpus

    def totalRunningGpus(self):
        """Returns the total number of running gpus in the allocation.

        :rtype:  float
        :return: total number of running gpus in the allocation
        """
        return self.data.stats.running_gpus

    def totalLockedGpus(self):
        """Returns the total number of locked gpus in the allocation.

        :rtype:  float
        :return: total number of locked gpus in the allocation
        """
        return self.data.stats.locked_gpus

    def totalHosts(self):
        """Returns the total number of hosts in the allocation.

        :rtype:  int
        :return: total number of hosts in the allocation
        """
        return self.data.stats.hosts

    def totalLockedHosts(self):
        """Returns the total number of locked hosts in the allocation.

        :rtype:  int
        :return: total number of locked hosts in the allocation
        """
        return self.data.stats.locked_hosts

    def totalDownHosts(self):
        """Returns the total number of down hosts in the allocation.

        :rtype:  int
        :return: total number of down hosts in the allocation
        """
        return self.data.stats.down_hosts



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.task`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import task_pb2
import opencue.wrappers.task


@mock.patch('opencue.cuebot.Cuebot.getStub')
class TaskTests(unittest.TestCase):

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = task_pb2.TaskDeleteResponse()
        getStubMock.return_value = stubMock

        task = opencue.wrappers.task.Task(
            task_pb2.Task(name='testTask'))
        task.delete()

        stubMock.Delete.assert_called_with(
            task_pb2.TaskDeleteRequest(task=task.data), timeout=mock.ANY)

    def testSetMinCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Save.return_value = task_pb2.TaskSetMinCoresResponse()
        getStubMock.return_value = stubMock

        minCores = 10

        task = opencue.wrappers.task.Task(
            task_pb2.Task(name='testTask'))
        task.setMinCores(minCores)

        stubMock.SetMinCores.assert_called_with(
            task_pb2.TaskSetMinCoresRequest(task=task.data, new_min_cores=minCores),
            timeout=mock.ANY)


if __name__ == '__main__':
    unittest.main()
